@@ -1,6 +1,27 @@
+// Copyright Joyent, Inc. and other Node contributors.
+//
+// Permission is hereby granted, free of charge, to any person obtaining a
+// copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to permit
+// persons to whom the Software is furnished to do so, subject to the
+// following conditions:
+//
+// The above copyright notice and this permission notice shall be included
+// in all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
+// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
+// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
+// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
+// USE OR OTHER DEALINGS IN THE SOFTWARE.
+
 var util = require('util');
 var EventEmitter = require('events').EventEmitter;
-var Stream = require('net').Stream;
+var Stream = require('net_legacy').Stream; // FIXME
 var InternalChildProcess = process.binding('child_process').ChildProcess;
 var constants;
 
@@ -11,6 +32,60 @@ var spawn = exports.spawn = function(path, args /*, options, customFds */) {
   return child;
 };
 
+
+function setupChannel(target, fd) {
+  target._channel = new Stream(fd);
+  target._channel.writable = true;
+  target._channel.readable = true;
+
+  target._channel.resume();
+  target._channel.setEncoding('utf8');
+
+  var buffer = '';
+  target._channel.on('data', function(d) {
+    buffer += d;
+    var i;
+    while ((i = buffer.indexOf('\n')) >= 0) {
+      var json = buffer.slice(0, i);
+      buffer = buffer.slice(i + 1);
+      var m = JSON.parse(json);
+      target.emit('message', m);
+    }
+  });
+
+  target.send = function(m) {
+    target._channel.write(JSON.stringify(m) + '\n');
+  };
+}
+
+
+exports.fork = function(modulePath, args, options) {
+  if (!options) options = {};
+  options.wantChannel = true;
+
+  if (!args) args = [];
+  args.unshift(modulePath);
+
+  // Unless they gave up customFds, just use the parent process
+  if (!options.customFds) options.customFds = [0, 1, 2];
+
+  var child = spawn(process.execPath, args, options);
+
+  setupChannel(child, child.fds[3]);
+
+  child.on('exit', function() {
+    child._channel.destroy();
+  });
+
+  return child;
+};
+
+
+exports._forkChild = function(fd) {
+  setupChannel(process, fd);
+};
+
+
 exports.exec = function(command /*, options, callback */) {
   var _slice = Array.prototype.slice;
   var args = ['/bin/sh', ['-c', command]].concat(_slice.call(arguments, 1));
@@ -215,11 +290,16 @@ ChildProcess.prototype.spawn = function(path, args, options, customFds) {
 
   var envPairs = [];
   var keys = Object.keys(env);
-  for (var index = 0, keysLength = keys.length; index < keysLength; index++) {
-    var key = keys[index];
+  for (var key in env) {
     envPairs.push(key + '=' + env[key]);
   }
 
+  if (options && options.wantChannel) {
+    // The FILLMEIN will be replaced in C land with an integer!
+    // AWFUL! :D
+    envPairs.push('NODE_CHANNEL_FD=FILLMEIN');
+  }
+
   var fds = this._internal.spawn(path,
                                  args,
                                  cwd,
