@@ -29,6 +29,7 @@ try {
   var Decipher = binding.Decipher;
   var Sign = binding.Sign;
   var Verify = binding.Verify;
+  var DiffieHellman = binding.DiffieHellman;
   var crypto = true;
 } catch (e) {
 
@@ -36,7 +37,7 @@ try {
 }
 
 
-function Credentials(secureProtocol) {
+function Credentials(secureProtocol, flags, context) {
   if (!(this instanceof Credentials)) {
     return new Credentials(secureProtocol);
   }
@@ -45,27 +46,39 @@ function Credentials(secureProtocol) {
     throw new Error('node.js not compiled with openssl crypto support.');
   }
 
-  this.context = new SecureContext();
-
-  if (secureProtocol) {
-    this.context.init(secureProtocol);
+  if (context) {
+    this.context = context;
   } else {
-    this.context.init();
+    this.context = new SecureContext();
+
+    if (secureProtocol) {
+      this.context.init(secureProtocol);
+    } else {
+      this.context.init();
+    }
   }
 
+  if (flags) this.context.setOptions(flags);
 }
 
 exports.Credentials = Credentials;
 
 
-exports.createCredentials = function(options) {
+exports.createCredentials = function(options, context) {
   if (!options) options = {};
-  var c = new Credentials(options.secureProtocol);
+
+  var c = new Credentials(options.secureProtocol,
+                          options.secureOptions,
+                          context);
+
+  if (context) return c;
 
   if (options.key) c.context.setKey(options.key);
 
   if (options.cert) c.context.setCert(options.cert);
 
+  if (options.ciphers) c.context.setCiphers(options.ciphers);
+
   if (options.ca) {
     if (Array.isArray(options.ca)) {
       for (var i = 0, len = options.ca.length; i < len; i++) {
@@ -135,3 +148,15 @@ exports.Verify = Verify;
 exports.createVerify = function(algorithm) {
   return (new Verify).init(algorithm);
 };
+
+exports.DiffieHellman = DiffieHellman;
+exports.createDiffieHellman = function(size_or_key, enc) {
+  if (!size_or_key) {
+    return new DiffieHellman();
+  } else if (!enc) {
+    return new DiffieHellman(size_or_key);
+  } else {
+    return new DiffieHellman(size_or_key, enc);
+  }
+
+}
