@@ -105,6 +105,9 @@ EventEmitter.prototype.addListener = function(type, listener) {
     this._events[type] = listener;
   } else if (isArray(this._events[type])) {
 
+    // If we've already got an array, just append.
+    this._events[type].push(listener);
+
     // Check for listener leak
     if (!this._events[type].warned) {
       var m;
@@ -123,9 +126,6 @@ EventEmitter.prototype.addListener = function(type, listener) {
         console.trace();
       }
     }
-
-    // If we've already got an array, just append.
-    this._events[type].push(listener);
   } else {
     // Adding the second element, need to change to array.
     this._events[type] = [this._events[type], listener];
@@ -137,6 +137,10 @@ EventEmitter.prototype.addListener = function(type, listener) {
 EventEmitter.prototype.on = EventEmitter.prototype.addListener;
 
 EventEmitter.prototype.once = function(type, listener) {
+  if ('function' !== typeof listener) {
+    throw new Error('.once only takes instances of Function');
+  }
+
   var self = this;
   function g() {
     self.removeListener(type, g);
@@ -184,6 +188,11 @@ EventEmitter.prototype.removeListener = function(type, listener) {
 };
 
 EventEmitter.prototype.removeAllListeners = function(type) {
+  if (arguments.length === 0) {
+    this._events = {};
+    return this;
+  }
+
   // does not use listeners(), so no side effect of creating _events[type]
   if (type && this._events && this._events[type]) this._events[type] = null;
   return this;
