@@ -432,6 +432,33 @@ fs.unlinkSync = function(path) {
   return binding.unlink(path);
 };
 
+fs.fchmod = function(fd, mode, callback) {
+  binding.fchmod(fd, modeNum(mode), callback || noop);
+};
+
+fs.fchmodSync = function(fd, mode) {
+  return binding.fchmod(fd, modeNum(mode));
+};
+
+if (constants.hasOwnProperty('O_SYMLINK')) {
+  fs.lchmod = function(path, mode, callback) {
+    callback = callback || noop;
+    fs.open(path, constants.O_WRONLY | constants.O_SYMLINK, function (err, fd) {
+      if (err) {
+        callback(err);
+        return;
+      }
+      fs.fchmod(fd, mode, callback);
+    });
+  };
+
+  fs.lchmodSync = function(path, mode) {
+    var fd = fs.openSync(path, constants.O_WRONLY | constants.O_SYMLINK);
+    return fs.fchmodSync(fd, mode);
+  };
+}
+
+
 fs.chmod = function(path, mode, callback) {
   binding.chmod(path, modeNum(mode), callback || noop);
 };
@@ -440,6 +467,32 @@ fs.chmodSync = function(path, mode) {
   return binding.chmod(path, modeNum(mode));
 };
 
+if (constants.hasOwnProperty('O_SYMLINK')) {
+  fs.lchown = function(path, uid, gid, callback) {
+    callback = callback || noop;
+    fs.open(path, constants.O_WRONLY | constants.O_SYMLINK, function (err, fd) {
+      if (err) {
+        callback(err);
+        return;
+      }
+      fs.fchown(fd, uid, gid, callback);
+    });
+  };
+
+  fs.lchownSync = function(path, uid, gid) {
+    var fd = fs.openSync(path, constants.O_WRONLY | constants.O_SYMLINK);
+    return fs.fchownSync(fd, uid, gid);
+  };
+}
+
+fs.fchown = function(fd, uid, gid, callback) {
+  binding.fchown(fd, uid, gid, callback || noop);
+};
+
+fs.fchownSync = function(fd, uid, gid) {
+  return binding.fchown(fd, uid, gid);
+};
+
 fs.chown = function(path, uid, gid, callback) {
   binding.chown(path, uid, gid, callback || noop);
 };
@@ -448,6 +501,45 @@ fs.chownSync = function(path, uid, gid) {
   return binding.chown(path, uid, gid);
 };
 
+// converts Date or number to a fractional UNIX timestamp
+function toUnixTimestamp(time) {
+  if (typeof time == 'number') {
+    return time;
+  }
+  if (time instanceof Date) {
+    // convert to 123.456 UNIX timestamp
+    return time.getTime() / 1000;
+  }
+  throw new Error("Cannot parse time: " + time);
+}
+
+// exported for unit tests, not for public consumption
+fs._toUnixTimestamp = toUnixTimestamp;
+
+fs.utimes = function(path, atime, mtime, callback) {
+  atime = toUnixTimestamp(atime);
+  mtime = toUnixTimestamp(mtime);
+  binding.utimes(path, atime, mtime, callback || noop);
+};
+
+fs.utimesSync = function(path, atime, mtime) {
+  atime = toUnixTimestamp(atime);
+  mtime = toUnixTimestamp(mtime);
+  binding.utimes(path, atime, mtime);
+};
+
+fs.futimes = function(fd, atime, mtime, callback) {
+  atime = toUnixTimestamp(atime);
+  mtime = toUnixTimestamp(mtime);
+  binding.futimes(fd, atime, mtime, callback || noop);
+};
+
+fs.futimesSync = function(fd, atime, mtime) {
+  atime = toUnixTimestamp(atime);
+  mtime = toUnixTimestamp(mtime);
+  binding.futimes(fd, atime, mtime);
+};
+
 function writeAll(fd, buffer, offset, length, callback) {
   // write(fd, buffer, offset, length, position, callback)
   fs.write(fd, buffer, offset, length, offset, function(writeErr, written) {
@@ -542,7 +634,7 @@ var path = require('path'),
     isWindows = process.platform === 'win32';
 
 if (isWindows) {
-  // Node doesn't support symlinks / lstat on windows. Hence realpatch is just
+  // Node doesn't support symlinks / lstat on windows. Hence realpath is just
   // the same as path.resolve that fails if the path doesn't exists.
 
   // windows version
@@ -799,11 +891,12 @@ var ReadStream = fs.ReadStream = function(path, options) {
 
   if (this.encoding) this.setEncoding(this.encoding);
 
-  if (this.start !== undefined || this.end !== undefined) {
-    if (this.start === undefined || this.end === undefined) {
-      this.emit('error', new Error('Both start and end are needed ' +
-                                   'for range streaming.'));
-    } else if (this.start > this.end) {
+  if (this.start !== undefined) {
+    if (this.end === undefined) {
+      this.end = Infinity;
+    }
+
+    if (this.start > this.end) {
       this.emit('error', new Error('start must be <= end'));
     } else {
       this._firstRead = true;
@@ -979,6 +1072,7 @@ var WriteStream = fs.WriteStream = function(path, options) {
   this.flags = 'w';
   this.encoding = 'binary';
   this.mode = parseInt('0666', 8);
+  this.bytesWritten = 0;
 
   options = options || {};
 
@@ -1030,6 +1124,10 @@ WriteStream.prototype.flush = function() {
       return;
     }
 
+    if (method == fs.write) {
+      self.bytesWritten += arguments[1];
+    }
+
     // stop flushing after close
     if (method === fs.close) {
       if (cb) {
@@ -1061,7 +1159,8 @@ WriteStream.prototype.flush = function() {
 
 WriteStream.prototype.write = function(data) {
   if (!this.writable) {
-    throw new Error('stream not writable');
+    this.emit("error", new Error('stream not writable'));
+    return false;
   }
 
   this.drainable = true;
