@@ -284,6 +284,7 @@ IncomingMessage.prototype._addHeaderLine = function(field, value) {
     case 'connection':
     case 'cookie':
     case 'pragma':
+    case 'link':
       if (field in dest) {
         dest[field] += ', ' + value;
       } else {
@@ -770,6 +771,8 @@ function ServerResponse(req) {
     this.useChunkedEncodingByDefault = false;
     this.shouldKeepAlive = false;
   }
+
+  this._httpVersion = req.httpVersion;
 }
 util.inherits(ServerResponse, OutgoingMessage);
 
@@ -830,8 +833,9 @@ ServerResponse.prototype.writeHead = function(statusCode) {
     headers = obj;
   }
 
-  var statusLine = 'HTTP/1.1 ' + statusCode.toString() + ' ' +
-                   reasonPhrase + CRLF;
+  var statusLine = 'HTTP/' + this._httpVersion
+                 + ' ' + statusCode.toString()
+                 + ' ' + reasonPhrase + CRLF;
 
   if (statusCode === 204 || statusCode === 304 ||
       (100 <= statusCode && statusCode <= 199)) {
@@ -976,20 +980,12 @@ function connectionListener(socket) {
   var self = this;
   var outgoing = [];
   var incoming = [];
-  var abortError = null;
 
   function abortIncoming() {
-    if (!abortError) {
-      abortError = new Error('http.ServerRequest was aborted by the client');
-      abortError.code = 'aborted';
-    }
-
     while (incoming.length) {
       var req = incoming.shift();
-
-      // @deprecated, should be removed in 0.5.x
       req.emit('aborted');
-      req.emit('close', abortError);
+      req.emit('close');
     }
     // abort socket._httpMessage ?
   }
@@ -1000,8 +996,6 @@ function connectionListener(socket) {
 
   socket.setTimeout(2 * 60 * 1000); // 2 minute timeout
   socket.addListener('timeout', function() {
-    abortError = new Error('http.ServerRequest timed out');
-    abortError.code = 'timeout';
     socket.destroy();
   });
 
@@ -1132,10 +1126,12 @@ function Agent(options) {
   this.options = options;
   this.host = options.host;
   this.port = options.port || this.defaultPort;
+  this.socketPath = options.socketPath;
 
   this.queue = [];
   this.sockets = [];
   this.maxSockets = Agent.defaultMaxSockets;
+
 }
 util.inherits(Agent, EventEmitter);
 exports.Agent = Agent;
@@ -1151,12 +1147,6 @@ Agent.prototype.appendMessage = function(options) {
   this.queue.push(req);
   req._queue = this.queue;
 
-  /*
-  req.on('finish', function () {
-    self._cycle();
-  });
-  */
-
   this._cycle();
 
   return req;
@@ -1175,7 +1165,7 @@ Agent.prototype._establishNewConnection = function() {
 
   // Grab a new "socket". Depending on the implementation of _getConnection
   // this could either be a raw TCP socket or a TLS stream.
-  var socket = this._getConnection(this.host, this.port, function() {
+  var socket = this._getConnection(self, function() {
     socket._httpConnecting = false;
     self.emit('connect'); // mostly for the shim.
     debug('Agent _getConnection callback');
@@ -1355,9 +1345,18 @@ Agent.prototype._establishNewConnection = function() {
 
 // Sub-classes can overwrite this method with e.g. something that supplies
 // TLS streams.
-Agent.prototype._getConnection = function(host, port, cb) {
+Agent.prototype._getConnection = function(options, cb) {
   debug('Agent connected!');
-  var c = net.createConnection(port, host);
+
+  var c;
+
+  if (options.host) {
+    c = net.createConnection(options.port, options.host);
+  } else if (options.socketPath) {
+    c = net.createConnection(options.socketPath);
+  } else {
+    c = net.createConnection(options.port);
+  }
   c.on('connect', cb);
   return c;
 };
@@ -1373,8 +1372,6 @@ Agent.prototype._cycle = function() {
   var first = this.queue[0];
   if (!first) return;
 
-  var haveConnectingSocket = false;
-
   // First try to find an available socket.
   for (var i = 0; i < this.sockets.length; i++) {
     var socket = this.sockets[i];
@@ -1394,13 +1391,11 @@ Agent.prototype._cycle = function() {
       self._cycle(); // try to dispatch another
       return;
     }
-
-    if (socket._httpConnecting) haveConnectingSocket = true;
   }
 
   // If no sockets are connecting, and we have space for another we should
   // be starting a new connection to handle this request.
-  if (!haveConnectingSocket && this.sockets.length < this.maxSockets) {
+  if (this.sockets.length < this.maxSockets) {
     this._establishNewConnection();
   }
 
@@ -1417,14 +1412,41 @@ Agent.prototype._cycle = function() {
 // to remove it?
 var agents = {};
 
+// Backwards compatible with legacy getAgent(host, port);
+function getAgent(options) {
+  var agent;
+  var host;
+  var id;
+  var port;
 
-function getAgent(host, port) {
-  port = port || 80;
-  var id = host + ':' + port;
-  var agent = agents[id];
+  var _opts = {};
+
+  if (typeof options === 'string') {
+    port = arguments[1] || 80;
+    id = options + ':' + port;
+    _opts.host = options;
+    _opts.port = port;
+  } else if (options && typeof options === 'object') {
+    if (options.port || options.host) {
+      host = options.host || 'localhost';
+      port = options.port || 80;
+      id = host + ':' + port;
+      _opts.host = host;
+      _opts.port = port;
+    } else if (options.socketPath) {
+      id = options.socketPath;
+      _opts.socketPath = options.socketPath;
+    } else {
+      throw new TypeError('Invalid options specification to getAgent');
+    }
+  } else {
+    throw new TypeError('Invalid argument to getAgent');
+  }
+
+  agent = agents[id];
 
   if (!agent) {
-    agent = agents[id] = new Agent({ host: host, port: port });
+    agent = agents[id] = new Agent(_opts);
   }
 
   return agent;
@@ -1442,7 +1464,7 @@ exports._requestFromAgent = function(options, cb) {
 
 exports.request = function(options, cb) {
   if (options.agent === undefined) {
-    options.agent = getAgent(options.host, options.port);
+    options.agent = getAgent(options);
   } else if (options.agent === false) {
     options.agent = new Agent(options);
   }
@@ -1679,6 +1701,8 @@ exports.cat = function(url, encoding_, headers_) {
       headers = {},
       callback = null;
 
+  console.error("http.cat will be removed in the near future. use http.get");
+
   // parse the arguments for the various options... very ugly
   if (typeof(arguments[1]) == 'string') {
     encoding = arguments[1];
