@@ -20,7 +20,7 @@
 // USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 var NativeModule = require('native_module');
-var Script = process.binding('evals').Script;
+var Script = process.binding('evals').NodeScript;
 var runInThisContext = Script.runInThisContext;
 var runInNewContext = Script.runInNewContext;
 var assert = require('assert').ok;
@@ -199,12 +199,7 @@ Module._nodeModulePaths = function(from) {
   var paths = [];
   var parts = from.split(splitRe);
 
-  var root = parts.indexOf('node_modules') - 1;
-  if (root < 0) root = 0;
-
-  var tip = parts.length - 1;
-
-  for (var tip = parts.length - 1; tip >= root; tip --) {
+  for (var tip = parts.length - 1; tip >= 0; tip --) {
     // don't search in .../node_modules/node_modules
     if (parts[tip] === 'node_modules') continue;
     var dir = parts.slice(0, tip + 1).concat('node_modules').join(joiner);
@@ -365,6 +360,7 @@ Module.prototype._compile = function(content, filename) {
   require.cache = Module._cache;
 
   var dirname = path.dirname(filename);
+  var define = makeDefine(require, self);
 
   if (Module._contextLoad) {
     if (self.id !== '.') {
@@ -381,6 +377,7 @@ Module.prototype._compile = function(content, filename) {
       sandbox.module = self;
       sandbox.global = sandbox;
       sandbox.root = root;
+      sandbox.define = define;
 
       return runInNewContext(content, sandbox, filename, true);
     }
@@ -392,6 +389,7 @@ Module.prototype._compile = function(content, filename) {
     global.__filename = filename;
     global.__dirname = dirname;
     global.module = self;
+    global.define = define;
 
     return runInThisContext(content, filename, true);
   }
@@ -403,10 +401,44 @@ Module.prototype._compile = function(content, filename) {
   if (filename === process.argv[1] && global.v8debug) {
     global.v8debug.Debug.setBreakPoint(compiledWrapper, 0, 0);
   }
-  var args = [self.exports, require, self, filename, dirname];
+
+  var args = [self.exports, require, self, filename, dirname, define];
   return compiledWrapper.apply(self.exports, args);
 };
 
+// AMD compatibility
+function makeDefine(require, module) {
+  var called = false;
+  function define() {
+    if (called) {
+      throw new Error("define() may only be called once.");
+    }
+    called = true;
+
+    // only care about the last argument
+    var cb = arguments[ arguments.length - 1 ];
+
+    // set exports immediately:
+    // define({ foo: "bar" })
+    if (typeof cb !== 'function') {
+      module.exports = cb;
+      return;
+    }
+
+    var ret = cb(require, module.exports, module);
+
+    if (ret !== undefined) {
+      // set exports with return statement
+      // define(function () { return { foo: "bar" } })
+      module.exports = ret;
+    }
+  }
+
+  return define;
+}
+
+
+
 // Native extension for .js
 Module._extensions['.js'] = function(module, filename) {
   var content = NativeModule.require('fs').readFileSync(filename, 'utf8');
