@@ -60,27 +60,14 @@ module.filename = process.cwd() + '/repl';
 // hack for repl require to work properly with node_modules folders
 module.paths = require('module')._nodeModulePaths(module.filename);
 
-
-function resetContext() {
-  context = vm.createContext();
-  for (var i in global) context[i] = global[i];
-  context.module = module;
-  context.require = require;
-  context.global = context;
-  context.global.global = context;
-  for (var i in require.cache) delete require.cache[i];
-}
-
-
 // Can overridden with custom print functions, such as `probe` or `eyes.js`
 exports.writer = util.inspect;
 
 
 function REPLServer(prompt, stream) {
   var self = this;
-  if (!context) resetContext();
-  if (!exports.repl) exports.repl = this;
-  self.context = context;
+
+  self.resetContext();
   self.bufferedCommand = '';
 
   if (stream) {
@@ -93,7 +80,7 @@ function REPLServer(prompt, stream) {
     process.stdin.resume();
   }
 
-  self.prompt = prompt || '> ';
+  self.prompt = (prompt != undefined ? prompt : '> ');
 
   function complete(text) {
     return self.complete(text);
@@ -105,7 +92,7 @@ function REPLServer(prompt, stream) {
   this.commands = {};
   defineDefaultCommands(this);
 
-  if (rli.enabled && !disableColors) {
+  if (rli.enabled && !disableColors && exports.writer === util.inspect) {
     // Turn on ANSI coloring.
     exports.writer = function(obj, showHidden, depth) {
       return util.inspect(obj, showHidden, depth, true);
@@ -114,17 +101,27 @@ function REPLServer(prompt, stream) {
 
   rli.setPrompt(self.prompt);
 
+  var sawSIGINT = false;
   rli.on('SIGINT', function() {
-    if (self.bufferedCommand && self.bufferedCommand.length > 0) {
-      rli.write('\n');
-      self.bufferedCommand = '';
-      self.displayPrompt();
-    } else {
-      rli.close();
+    if (sawSIGINT) {
+       rli.close();
+       process.exit();
     }
+    var bareInt = false;
+    if (!(self.bufferedCommand && self.bufferedCommand.length > 0) &&
+        rli.line.length === 0) {
+      rli.write('\n(^C again to quit)');
+      bareInt = true;
+    }
+    rli.line = '';
+    rli.write('\n');
+    self.bufferedCommand = '';
+    self.displayPrompt();
+    sawSIGINT = bareInt;
   });
 
   rli.addListener('line', function(cmd) {
+    sawSIGINT = false;
     var skipCatchall = false;
     cmd = trimWhitespace(cmd);
 
@@ -159,7 +156,7 @@ function REPLServer(prompt, stream) {
             // First we attempt to eval as expression with parens.
             // This catches '{a : 1}' properly.
             ret = vm.runInContext('(' + self.bufferedCommand + ')',
-                                  context,
+                                  self.context,
                                   'repl');
             if (typeof ret !== 'function') success = true;
           } catch (e) {
@@ -168,11 +165,11 @@ function REPLServer(prompt, stream) {
 
           if (!success) {
             // Now as statement without parens.
-            ret = vm.runInContext(self.bufferedCommand, context, 'repl');
+            ret = vm.runInContext(self.bufferedCommand, self.context, 'repl');
           }
 
           if (ret !== undefined) {
-            context._ = ret;
+            self.context._ = ret;
             self.outputStream.write(exports.writer(ret) + '\n');
           }
 
@@ -184,8 +181,8 @@ function REPLServer(prompt, stream) {
           // It could also be an error from JSON.parse
           } else if (e &&
                      e.stack &&
-                     e.stack.match('Unexpected token ILLEGAL') &&
-                     e.stack.match(/Object.parse \(native\)/)) {
+                     e.stack.match(/^SyntaxError: Unexpected token .*\n/) &&
+                     e.stack.match(/\n    at Object.parse \(native\)\n/)) {
             throw e;
           }
         }
@@ -215,10 +212,33 @@ exports.REPLServer = REPLServer;
 // prompt is a string to print on each line for the prompt,
 // source is a stream to use for I/O, defaulting to stdin/stdout.
 exports.start = function(prompt, source) {
-  return new REPLServer(prompt, source);
+  var repl = new REPLServer(prompt, source);
+  if (!exports.repl) exports.repl = repl;
+  return repl;
 };
 
 
+REPLServer.prototype.createContext = function() {
+  var context = vm.createContext();
+
+  for (var i in global) context[i] = global[i];
+  context.module = module;
+  context.require = require;
+  context.global = context;
+  context.global.global = context;
+
+  return context;
+};
+
+REPLServer.prototype.resetContext = function(force) {
+  if (!context || force) {
+    context = this.createContext();
+    for (var i in require.cache) delete require.cache[i];
+  }
+
+  this.context = context;
+};
+
 REPLServer.prototype.displayPrompt = function() {
   this.rli.setPrompt(this.bufferedCommand.length ? '... ' : this.prompt);
   this.rli.prompt();
@@ -503,7 +523,7 @@ function defineDefaultCommands(repl) {
     action: function() {
       this.outputStream.write('Clearing context...\n');
       this.bufferedCommand = '';
-      resetContext();
+      this.resetContext(true);
       this.displayPrompt();
     }
   });
