@@ -27,6 +27,8 @@ var stream = require('stream');
 var END_OF_FILE = 42;
 var assert = require('assert').ok;
 
+var NPN_ENABLED = process.binding('constants').NPN_ENABLED;
+
 var debug;
 if (process.env.NODE_DEBUG && /tls/.test(process.env.NODE_DEBUG)) {
   debug = function(a) { console.error('TLS:', a); };
@@ -38,10 +40,36 @@ if (process.env.NODE_DEBUG && /tls/.test(process.env.NODE_DEBUG)) {
 var Connection = null;
 try {
   Connection = process.binding('crypto').Connection;
+  exports.NPN_ENABLED = NPN_ENABLED;
 } catch (e) {
   throw new Error('node.js not compiled with openssl crypto support.');
 }
 
+// Convert protocols array into valid OpenSSL protocols list
+// ("\x06spdy/2\x08http/1.1\x08http/1.0")
+function convertNPNProtocols(NPNProtocols, out) {
+  // If NPNProtocols is Array - translate it into buffer
+  if (Array.isArray(NPNProtocols)) {
+    var buff = new Buffer(NPNProtocols.reduce(function(p, c) {
+      return p + 1 + Buffer.byteLength(c);
+    }, 0));
+
+    NPNProtocols.reduce(function(offset, c) {
+      var clen = Buffer.byteLength(c);
+      buff[offset] = clen;
+      buff.write(c, offset + 1);
+
+      return offset + 1 + clen;
+    }, 0);
+
+    NPNProtocols = buff;
+  }
+
+  // If it's already a Buffer - store it
+  if (Buffer.isBuffer(NPNProtocols)) {
+    out.NPNProtocols = NPNProtocols;
+  }
+};
 
 // Base class of both CleartextStream and EncryptedStream
 function CryptoStream(pair) {
@@ -449,12 +477,14 @@ EncryptedStream.prototype._pusher = function(pool, offset, length) {
  * Provides a pair of streams to do encrypted communication.
  */
 
-function SecurePair(credentials, isServer, requestCert, rejectUnauthorized) {
+function SecurePair(credentials, isServer, requestCert, rejectUnauthorized,
+                    NPNProtocols) {
   if (!(this instanceof SecurePair)) {
     return new SecurePair(credentials,
                           isServer,
                           requestCert,
-                          rejectUnauthorized);
+                          rejectUnauthorized,
+                          NPNProtocols);
   }
 
   var self = this;
@@ -488,6 +518,10 @@ function SecurePair(credentials, isServer, requestCert, rejectUnauthorized) {
                              this._requestCert,
                              this._rejectUnauthorized);
 
+  if (NPN_ENABLED && NPNProtocols) {
+    this.ssl.setNPNProtocols(NPNProtocols);
+    this.npnProtocol = null;
+  }
 
   /* Acts as a r/w stream to the cleartext side of the stream. */
   this.cleartext = new CleartextStream(this);
@@ -593,6 +627,9 @@ SecurePair.prototype.cycle = function(depth) {
 
 SecurePair.prototype.maybeInitFinished = function() {
   if (this.ssl && !this._secureEstablished && this.ssl.isInitFinished()) {
+    if (NPN_ENABLED) {
+      this.npnProtocol = this.ssl.getNegotiatedProtocol();
+    }
     this._secureEstablished = true;
     debug('secure established');
     this.emit('secure');
@@ -736,6 +773,7 @@ function Server(/* [options], listener */) {
     ca: self.ca,
     ciphers: self.ciphers,
     secureProtocol: self.secureProtocol,
+    secureOptions: self.secureOptions,
     crl: self.crl
   });
 
@@ -748,13 +786,15 @@ function Server(/* [options], listener */) {
     var pair = new SecurePair(creds,
                               true,
                               self.requestCert,
-                              self.rejectUnauthorized);
+                              self.rejectUnauthorized,
+                              self.NPNProtocols);
 
     var cleartext = pipe(pair, socket);
     cleartext._controlReleased = false;
 
     pair.on('secure', function() {
       pair.cleartext.authorized = false;
+      pair.cleartext.npnProtocol = pair.npnProtocol;
       if (!self.requestCert) {
         cleartext._controlReleased = true;
         self.emit('secureConnection', pair.cleartext, pair.encrypted);
@@ -812,6 +852,10 @@ Server.prototype.setOptions = function(options) {
   if (options.ca) this.ca = options.ca;
   if (options.secureProtocol) this.secureProtocol = options.secureProtocol;
   if (options.crl) this.crl = options.crl;
+  if (options.ciphers) this.ciphers = options.ciphers;
+  if (options.secureProtocol) this.secureProtocol = options.secureProtocol;
+  if (options.secureOptions) this.secureOptions = options.secureOptions;
+  if (options.NPNProtocols) convertNPNProtocols(options.NPNProtocols, this);
 };
 
 
@@ -854,7 +898,9 @@ exports.connect = function(port /* host, options, cb */) {
   var sslcontext = crypto.createCredentials(options);
   //sslcontext.context.setCiphers('RC4-SHA:AES128-SHA:AES256-SHA');
 
-  var pair = new SecurePair(sslcontext, false);
+  convertNPNProtocols(options.NPNProtocols, this);
+  var pair = new SecurePair(sslcontext, false, true, false,
+                            this.NPNProtocols);
 
   var cleartext = pipe(pair, socket);
 
@@ -863,6 +909,8 @@ exports.connect = function(port /* host, options, cb */) {
   pair.on('secure', function() {
     var verifyError = pair.ssl.verifyError();
 
+    cleartext.npnProtocol = pair.npnProtocol;
+
     if (verifyError) {
       cleartext.authorized = false;
       cleartext.authorizationError = verifyError;
