@@ -35,6 +35,9 @@
 #include <sys/wait.h>
 #endif
 
+#include <sys/socket.h> /* socketpair */
+#include <sys/un.h>
+
 # ifdef __APPLE__
 # include <crt_externs.h>
 # define environ (*_NSGetEnviron())
@@ -153,7 +156,7 @@ Handle<Value> ChildProcess::Spawn(const Arguments& args) {
   // Copy fourth argument, args[3], into a c-string array called env.
   Local<Array> env_handle = Local<Array>::Cast(args[3]);
   int envc = env_handle->Length();
-  char **env = new char*[envc+1]; // heap allocated to detect errors
+  char **env = new char*[envc + 1]; // heap allocated to detect errors
   env[envc] = NULL;
   for (int i = 0; i < envc; i++) {
     String::Utf8Value pair(env_handle->Get(Integer::New(i))->ToString());
@@ -206,7 +209,7 @@ Handle<Value> ChildProcess::Spawn(const Arguments& args) {
       String::New("setgid argument must be a number or a string")));
   }
 
-
+  int channel_fd = -1;
 
   int r = child->Spawn(argv[0],
                        argv,
@@ -218,7 +221,8 @@ Handle<Value> ChildProcess::Spawn(const Arguments& args) {
                        custom_uid,
                        custom_uname,
                        custom_gid,
-                       custom_gname);
+                       custom_gname,
+                       &channel_fd);
 
   if (custom_uname != NULL) free(custom_uname);
   if (custom_gname != NULL) free(custom_gname);
@@ -235,7 +239,8 @@ Handle<Value> ChildProcess::Spawn(const Arguments& args) {
     return ThrowException(Exception::Error(String::New("Error spawning")));
   }
 
-  Local<Array> a = Array::New(3);
+
+  Local<Array> a = Array::New(channel_fd >= 0 ? 4 : 3);
 
   assert(fds[0] >= 0);
   a->Set(0, Integer::New(fds[0])); // stdin
@@ -244,6 +249,10 @@ Handle<Value> ChildProcess::Spawn(const Arguments& args) {
   assert(fds[2] >= 0);
   a->Set(2, Integer::New(fds[2])); // stderr
 
+  if (channel_fd >= 0) {
+    a->Set(3, Integer::New(channel_fd));
+  }
+
   return scope.Close(a);
 }
 
@@ -291,6 +300,8 @@ void ChildProcess::Stop() {
 // Note that args[0] must be the same as the "file" param.  This is an
 // execvp() requirement.
 //
+// TODO: The arguments are rediculously long. Needs to be put into a struct.
+//
 int ChildProcess::Spawn(const char *file,
                         char *const args[],
                         const char *cwd,
@@ -301,7 +312,8 @@ int ChildProcess::Spawn(const char *file,
                         int custom_uid,
                         char *custom_uname,
                         int custom_gid,
-                        char *custom_gname) {
+                        char *custom_gname,
+                        int* channel) {
   HandleScope scope;
   assert(pid_ == -1);
   assert(!ev_is_active(&child_watcher_));
@@ -332,11 +344,37 @@ int ChildProcess::Spawn(const char *file,
     SetCloseOnExec(stderr_pipe[1]);
   }
 
+
+  // The channel will be used by js-land "fork()" for a little JSON channel.
+  // The pointer is used to pass one end of the socket pair back to the
+  // parent.
+  // channel_fds[0] is for the parent
+  // channel_fds[1] is for the child
+  int channel_fds[2] = { -1, -1 };
+
+#define NODE_CHANNEL_FD "NODE_CHANNEL_FD"
+
+  for (int i = 0; env[i]; i++) {
+    if (!strncmp(env[i], NODE_CHANNEL_FD, sizeof NODE_CHANNEL_FD - 1)) {
+      if (socketpair(AF_UNIX, SOCK_STREAM, 0, channel_fds)) {
+        perror("socketpair()");
+        return -1;
+      }
+
+      assert(channel_fds[0] >= 0 && channel_fds[1] >= 0);
+
+      SetNonBlocking(channel_fds[0]);
+      SetNonBlocking(channel_fds[1]);
+      // Write over the FILLMEIN :D
+      sprintf(env[i], NODE_CHANNEL_FD "=%d", channel_fds[1]);
+    }
+  }
+
   // Save environ in the case that we get it clobbered
   // by the child process.
   char **save_our_env = environ;
 
-  switch (pid_ = vfork()) {
+  switch (pid_ = fork()) {
     case -1:  // Error.
       Stop();
       return -4;
@@ -429,7 +467,11 @@ int ChildProcess::Spawn(const char *file,
         _exit(127);
       }
 
-
+      // Close the parent's end of the channel.
+      if (channel_fds[0] >= 0) {
+        close(channel_fds[0]);
+        channel_fds[0] = -1;
+      }
 
       environ = env;
 
@@ -472,6 +514,17 @@ int ChildProcess::Spawn(const char *file,
     stdio_fds[2] = custom_fds[2];
   }
 
+  // Close the child's end of the channel.
+  if (channel_fds[1] >= 0) {
+    close(channel_fds[1]);
+    channel_fds[1] = -1;
+    assert(channel_fds[0] >= 0);
+    assert(channel);
+    *channel = channel_fds[0];
+  } else {
+    *channel = -1;
+  }
+
   return 0;
 }
 
