@@ -1,3 +1,24 @@
+// Copyright Joyent, Inc. and other Node contributors.
+//
+// Permission is hereby granted, free of charge, to any person obtaining a
+// copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to permit
+// persons to whom the Software is furnished to do so, subject to the
+// following conditions:
+//
+// The above copyright notice and this permission notice shall be included
+// in all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
+// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
+// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
+// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
+// USE OR OTHER DEALINGS IN THE SOFTWARE.
+
 var common = require('../common');
 var assert = require('assert');
 
@@ -108,12 +129,19 @@ function pingPongTest(port, host) {
 }
 
 /* All are run at once, so run on different ports */
-pingPongTest(20989, 'localhost');
+if (!process.useUV) {
+  // these tests will not run yet with net_uv TODO: remove when net_uv supports dns
+  pingPongTest(20997, '::1');
+  pingPongTest('/tmp/pingpong.sock');
+}
 pingPongTest(20988);
-pingPongTest(20997, '::1');
-pingPongTest('/tmp/pingpong.sock');
+pingPongTest(20989, 'localhost');
 
-process.addListener('exit', function() {
-  assert.equal(4, tests_run);
+process.addListener('exit', function () {
+  if (!process.useUV) {
+    assert.equal(4, tests_run);
+  } else {
+    assert.equal(2, tests_run);
+  }
   console.log('done');
 });
